/* Copyright (c) 1990 by Steve Peltz */

#include "MacPAD.h"
#include <DialogMgr.h>
#include <StdFilePkg.h>
#include <SegmentLdr.h>
#include <FileMgr.h>

#define PADFileType	'PLTO'
#define PADFileOwner	'PLTO'
#define PADIdent	'PLTO'
#define PADVersion	1

#define errAlert	128
#define confDialog	129
#define baudItem	3
#define flowItem	8
#define typeItem	9
#define wideItem	10
#define shrinkItem	12
#define highItem	13
#define titleItem	16
#define saveLocItem	17
#define resetLocItem	18

#define askAlert	130
#define aboutDialog	131
#define diskAlert	132

#define BoxLocX		60
#define BoxLocY		108

void	CopyPString(Str255 *, Str255 *);
Boolean	CheckNumeric(Str255 *);
Boolean	OpenConfigFile(Str255 *, short);
void	DiskError(char *);

static	Boolean		ChangedConfig,	/* config has been changed, so ask if should save */
			NewConfig;	/* config is default, so no file name yet */


/*----------------------------------------------*
 *	About					*
 *						*
 *	Display the "About ..." box		*
 *	Maybe someday have some help available.	*
 *----------------------------------------------*/

void	About()
{
	DialogPtr	myDialog;
	short		theItem;

	myDialog = GetNewDialog(aboutDialog, NIL, -1);
	ModalDialog(NIL, &theItem);
	DisposDialog(myDialog);
}


/*----------------------------------------------*
 *	Configure				*
 *						*
 *	Bring up Configure dialog box. Check	*
 *	for changed values, set proper flags	*
 *	for file save operations later.		*
 *----------------------------------------------*/

void	Configure()
{
	DialogPtr	myDialog;
	short		theItem;
	ConfigRec	nConfig;
	Boolean		newSize,	/* flag whether window size has changed */
			newType,	/* flag whether window type has changed */
			titleValue,	/* title bar selected */
			shrinkValue,	/* true if square proportions */
			saveValue,	/* save screen location */
			resetValue,	/* if resetting screen location */
			sameLoc,	/* current location same as config */
			resetLoc,	/* config location already reset */
			sameType;	/* title selection unchanged */
	short		wideValue,	/* selector for width button */
			highValue;	/* selector for height */
	Handle		baudHandle,	/* handles for different controls */
			flowHandle,
			typeHandle,
			wideHandle,
			shrinkHandle,
			highHandle,
			titleHandle,
			saveHandle,
			resetHandle;
	short		itemType;	/* ignored parameter for GetDItem */
	Rect		itemRect;	/* another ignored parameter */
	Str255		typeString;
	long		typeNum;

	nConfig = Config;	/* work in copy until accepted */
	newSize = FALSE;	/* check for new size selected */
	myDialog = GetNewDialog(confDialog, NIL, Front);
	GetDItem(myDialog, nConfig.BaudRate + baudItem, &itemType, &baudHandle, &itemRect);
	SetCtlValue(baudHandle, 1);	/* turn on radio button for current rate */
	GetDItem(myDialog, flowItem, &itemType, &flowHandle, &itemRect);
	SetCtlValue(flowHandle, nConfig.FlowControl);	/* set check box for flow control */
	GetDItem(myDialog, typeItem, &itemType, &typeHandle, &itemRect);
	typeNum = nConfig.SubType;
	NumToString(typeNum, &typeString);
	SetIText(typeHandle, &typeString);	/* Set and select the subtype field */
	SelIText(myDialog, typeItem, 0, 0x7fff);
	wideValue = (nConfig.PLATOSize.x == 640);
	GetDItem(myDialog, wideItem + wideValue, &itemType, &wideHandle, &itemRect);
	SetCtlValue(wideHandle, 1);
	if (nConfig.ScreenSize.v == 320)
		highValue = 2;
	else if (nConfig.ScreenSize.v == 448)
		highValue = 1;
	else
		highValue = 0;
	GetDItem(myDialog, shrinkItem, &itemType, &shrinkHandle, &itemRect);
	if (highValue) {
		shrinkValue = (nConfig.ScreenSize.h != nConfig.PLATOSize.x);
		SetCtlValue(shrinkHandle, shrinkValue);
	}
	else {
		shrinkValue = FALSE;
		HiliteControl(shrinkHandle, 255);
	}
		
	GetDItem(myDialog, highItem + highValue, &itemType, &highHandle, &itemRect);
	SetCtlValue(highHandle, 1);
	GetDItem(myDialog, titleItem, &itemType, &titleHandle, &itemRect);
	titleValue = (WindowType != plainWindow);
	sameType = (nConfig.WindowType == WindowType);
	newType = FALSE;
	SetCtlValue(titleHandle, titleValue);
	GetDItem(myDialog, saveLocItem, &itemType, &saveHandle, &itemRect);
	saveValue = FALSE;
	sameLoc = ((WindowLoc.v == nConfig.WindowLoc.v)
		&& (WindowLoc.h == nConfig.WindowLoc.h));
	resetLoc = ((nConfig.WindowLoc.v == 0) && (nConfig.WindowLoc.h == 0));
	if (sameLoc && sameType)
		HiliteControl(saveHandle, 255);
	GetDItem(myDialog, resetLocItem, &itemType, &resetHandle, &itemRect);
	resetValue = FALSE;
	if ((WindowLoc.v == 0) && (WindowLoc.h == 0))
		HiliteControl(resetHandle, 255);
	do {
		ModalDialog(NIL, &theItem);
		if ((theItem >= baudItem) && (theItem < flowItem)) {
			if ((theItem - baudItem) != nConfig.BaudRate) {
				nConfig.BaudRate = theItem - baudItem;
				SetCtlValue(baudHandle, 0);
				GetDItem(myDialog, theItem, &itemType, &baudHandle, &itemRect);
				SetCtlValue(baudHandle, 1);
			}
		}
		else if (theItem == flowItem) {
			nConfig.FlowControl = !nConfig.FlowControl;
			SetCtlValue(flowHandle, nConfig.FlowControl);
		}
		else if ((theItem >= wideItem) && (theItem < shrinkItem)) {
			if ((theItem - wideItem) != wideValue) {
				newSize = TRUE;
				wideValue = theItem - wideItem;
				SetCtlValue(wideHandle, 0);
				GetDItem(myDialog, theItem, &itemType, &wideHandle, &itemRect);
				SetCtlValue(wideHandle, 1);
			}
		}
		else if (theItem == shrinkItem) {
			newSize = TRUE;
			shrinkValue = !shrinkValue;
			SetCtlValue(shrinkHandle, shrinkValue);
		}
		else if ((theItem >= highItem) && (theItem < titleItem)) {
			if ((theItem - highItem) != highValue) {
				newSize = TRUE;
				if (!highValue)
					HiliteControl(shrinkHandle, 0);
				highValue = theItem - highItem;
				SetCtlValue(highHandle, 0);
				GetDItem(myDialog, theItem, &itemType, &highHandle, &itemRect);
				SetCtlValue(highHandle, 1);
				if (!highValue) {	/* no shrink if 512 high */
					shrinkValue = FALSE;
					SetCtlValue(shrinkHandle, 0);
					HiliteControl(shrinkHandle, 255);
				}
			}
		}
		else if (theItem == titleItem) {
			sameType = !sameType;
			titleValue = !titleValue;
			newType = TRUE;
			SetCtlValue(titleHandle, titleValue);
			if ((resetValue && resetLoc)
				|| (!resetValue && sameLoc))
				if (sameType) {
					saveValue = FALSE;
					SetCtlValue(saveHandle, 0);
					HiliteControl(saveHandle, 255);
				}
				else
					HiliteControl(saveHandle, 0);
		}
		else if (theItem == saveLocItem) {
			saveValue = !saveValue;
			SetCtlValue(saveHandle, saveValue);
		}
		else if (theItem == resetLocItem) {
			resetValue = !resetValue;
			SetCtlValue(resetHandle, resetValue);
			if (sameType)
				if ((resetValue && resetLoc)
					|| (!resetValue && sameLoc)) {
					saveValue = FALSE;
					SetCtlValue(saveHandle, 0);
					HiliteControl(saveHandle, 255);
				}
				else if ((!resetValue && resetLoc)
					|| (resetValue && sameLoc))
					HiliteControl(saveHandle, 0);
		}
		else if (theItem == OK) {
			GetIText(typeHandle, &typeString);	/* check for valid subtype */
			if (CheckNumeric(&typeString))
				StringToNum(&typeString, &typeNum);
			else
				typeNum = -1;		/* illegal value */
			if (typeNum != nConfig.SubType)
				if ((typeNum >= 0) && (typeNum <= 127)) {
					nConfig.SubType = typeNum;
					ChangedConfig = TRUE;
				}
				else {	/* redisplay current */
					typeNum = nConfig.SubType;
					NumToString(typeNum, &typeString);
					SetIText(typeHandle, &typeString);
					SelIText(myDialog, typeItem, 0, 0x7fff);
					theItem = 0;	/* prevent from leaving box */
					SysBeep(5);
				}
		}
	} while (theItem != OK && theItem != Cancel);
	DisposDialog(myDialog);
	if (theItem == OK) {
		if (Config.BaudRate != nConfig.BaudRate) {	/* change baud rate now */
			BaudRate(nConfig.BaudRate);
			ChangedConfig = TRUE;
		}

		if (newType) {
			newType = (titleValue == (WindowType == plainWindow));
			WindowType = titleValue ? titleWindow : plainWindow;
		}

		if (resetValue) {
			WindowLoc.v = 0;
			WindowLoc.h = 0;
		}

		if (saveValue) {
			nConfig.WindowLoc = WindowLoc;
			nConfig.WindowType = WindowType;
			ChangedConfig = TRUE;
		}

		if (newSize) {
			nConfig.PLATOSize.x = (wideValue) ? 640 : 512;
			nConfig.PLATOSize.y = 512;

			if (highValue == 2)
				nConfig.ScreenSize.v = 320;
			else if (highValue == 1)
				nConfig.ScreenSize.v = 448;
			else
				nConfig.ScreenSize.v = 512;

			if (shrinkValue)
				nConfig.ScreenSize.h = ((long)nConfig.PLATOSize.x *
					nConfig.ScreenSize.v) >> 9;
			else
				nConfig.ScreenSize.h = nConfig.PLATOSize.x;

			newSize = ((Config.ScreenSize.v != nConfig.ScreenSize.v)
				|| (Config.ScreenSize.h != nConfig.ScreenSize.h));
			ChangedConfig = ChangedConfig || newSize;
		}

		Config = nConfig;

		if (newType || resetValue || newSize)
			ResetPLATOWindow();
	}
}


/*----------------------------------------------*
 *	OpenFinderFile				*
 *						*
 *	Check for files passed by Finder (or	*
 *	equivalent). If it is a valid file,	*
 *	use it, else use default configuration.	*
 *	Open new window regardless.		*
 *----------------------------------------------*/

void	OpenFinderFile()
{
	short	openType,
		fileCount,
		fileIndex;
	AppFile	theFile;

	DefaultConfig();	/* initialize in case no valid files */
	CountAppFiles(&openType, &fileCount);
	for (fileIndex = 1; fileIndex <= fileCount; fileIndex++) {
		GetAppFiles(fileIndex, &theFile);
		if (theFile.fType == PADFileType) {
			if (OpenConfigFile(&theFile.fName, theFile.vRefNum)) {
				ClrAppFiles(fileIndex);
				break;
			}
		}
	}
}


/*----------------------------------------------*
 *	DefaultConfig				*
 *						*
 *	Create a default configuration record.	*
 *----------------------------------------------*/

void	DefaultConfig()
{
	Config.Ident = PADIdent;
	Config.Version = PADVersion;
	Config.BaudRate = 2;		/* 2400 baud */
	Config.WindowType = plainWindow;
	Config.WindowLoc.v = 0;
	Config.WindowLoc.h = 0;
	if (screenBits.bounds.bottom < 468)
		Config.ScreenSize.v = 320;
	else if (screenBits.bounds.bottom < 532)
		Config.ScreenSize.v = 448;
	else {
		Config.ScreenSize.v = 512;
		if (screenBits.bounds.bottom > 550)
			Config.WindowType = titleWindow;
	}
	Config.ScreenSize.h = 512;
	Config.PLATOSize.x = 512;
	Config.PLATOSize.y = 512;
	Config.DefaultForeColor.red = Config.DefaultForeColor.green =
		Config.DefaultForeColor.blue = 0x0000;
	Config.DefaultBackColor.red = Config.DefaultBackColor.green =
		Config.DefaultBackColor.blue = 0xffff;
	Config.CharLoadAddress = 0x3000;
	Config.TermType = 12;
	Config.SubType = 16;
	Config.LoadFile = 0;
	Config.Configuration = 0x40;	/* touch panel present */
	Config.FlowControl = FALSE;
	CopyPString((Str255 *)"\pUntitled", &ConfigFile);	/* window title uses this */
	WindowLoc = Config.WindowLoc;
	WindowType = Config.WindowType;
	NewConfig = TRUE;
	ChangedConfig = FALSE;
}


/*----------------------------------------------*
 *	OpenConfigFile				*
 *						*
 *	Open file and read Config. Check if	*
 *	valid file.				*
 *----------------------------------------------*/

Boolean	OpenConfigFile(fName, vRefNum)
	Str255 *	fName;
	short		vRefNum;
{
	short		fRefNum;
	long		count;
	ConfigRec	nConfig;
	OSErr		err;
	Boolean		valid;


	valid = FALSE;
	err = FSOpen(fName, vRefNum, &fRefNum);
	if ((err != noErr) && (err != opWrErr))
		DiskError("\pCan't open configuration file");
	else {
		count = sizeof(ConfigRec);
		err = FSRead(fRefNum, &count, &nConfig);
		if ((err == noErr) && (nConfig.Ident == PADIdent)
		&& (nConfig.Version == PADVersion)) {
			CopyPString(fName, &ConfigFile);
			ConfigVRefNum = vRefNum;
			Config = nConfig;
			WindowLoc = Config.WindowLoc;
			WindowType = Config.WindowType;
			NewConfig = FALSE;
			ChangedConfig = FALSE;
			valid = TRUE;
		}
		else if ((err == noErr) || (err == eofErr))
			DiskError("\pInvalid configuration file");
		else
			DiskError("\pCan't read configuration file");
	}
	err = FSClose(fRefNum);
	return (valid);
}


/*----------------------------------------------*
 *	GetConfigFile				*
 *						*
 *	Get file name from user and read	*
 *	Config from it.				*
 *----------------------------------------------*/
 
void	GetConfigFile()
{
	Point		where;
	SFTypeList	padType;
	SFReply		result;

	if (AskConfigFile()) {		/* save current config first if necessary */
		where.v = BoxLocY;
		where.h = BoxLocX;
		padType[0] = PADFileType;
		SFGetFile(where, NIL, NIL, 1, padType,
			NIL, &result);
		if (result.good)
			if (OpenConfigFile((Str255 *)&result.fName, result.vRefNum))
				ResetPLATOWindow();
	}
}


/*----------------------------------------------*
 *	SaveConfigFile				*
 *						*
 *	Save Config, either in current file	*
 *	or new file, depending on argument	*
 *----------------------------------------------*/

void	SaveConfigFile(ask)
	Boolean	ask;
{
	Point		where;
	SFReply		result;
	Str255 *	fileName;
	short		fRefNum;
	long		count;
	OSErr		err;

	if (ask || NewConfig) {
		if (NewConfig)
			fileName = (Str255 *)"";	/* no file name if new */
		else
			fileName = &ConfigFile;
		where.v = BoxLocY;
		where.h = BoxLocX;
		SFPutFile(where, "\pSave Configuration File as",
			fileName, NIL, &result);
		if (!result.good)
			return;		/* get out if cancelled */
		NewConfig = FALSE;	/* we now have a file name */
		CopyPString((Str255 *)&result.fName, &ConfigFile);
		ConfigVRefNum = result.vRefNum;
		SetWTitle(mainWindow, &ConfigFile);
		err = Create(&ConfigFile, ConfigVRefNum,
			PADFileOwner, PADFileType);
		if (err == dupFNErr) {
			err = FSDelete(&ConfigFile, ConfigVRefNum);
			if (err != noErr) {
				DiskError("\pCan't delete file");
				return;
			}
			err = Create(&ConfigFile, ConfigVRefNum,
				PADFileOwner, PADFileType);
		}
		if (err != noErr) {
			DiskError("\pCan't create file");
			return;
		}
	}
	err = FSOpen(ConfigFile, ConfigVRefNum, &fRefNum);
	count = sizeof(ConfigRec);
	err = FSWrite(fRefNum, &count, &Config);
	if (err != noErr)
		DiskError("\pCan't write file");
	else
		ChangedConfig = FALSE;	/* mark as actually saved */
	err = FSClose(fRefNum);
	err = FlushVol(NIL, ConfigVRefNum);
}


/*----------------------------------------------*
 *	AskFile					*
 *						*
 *	Ask user if he wants to save changed	*
 *	file, discard changes, or cancel the	*
 *	requested operation.			*
 *						*
 *	Returns TRUE if operation not		*
 *	cancelled.				*
 *----------------------------------------------*/

Boolean	AskConfigFile()
{
	short	theItem;

	if (ChangedConfig) {
		theItem = Alert(askAlert, NIL);
		if (theItem == Cancel)		/* cancel requested operation */
			return (FALSE);
		if (theItem != OK)	/* user doesn't want to save */
			return (TRUE);
		SaveConfigFile(FALSE);	/* resets ChangedConfig if successful */
	}
	return (!ChangedConfig);
}


/*----------------------------------------------*
 *	DiskError				*
 *						*
 *	Bring up alert box if there is a	*
 *	problem saving or reading a file.	*
 *----------------------------------------------*/

void	DiskError(err)
	char *	err;
{
	short	alertResult;

	ParamText(err, NIL, NIL, NIL);
	alertResult = StopAlert(diskAlert, NIL);
}
	

/*----------------------------------------------*
 *	Check					*
 *						*
 *	Check is called to check for various	*
 *	errors. If the passed value indicates	*
 *	an error, a dialog box is opened giving	*
 *	an option to continue or quit, and a	*
 *	message giving the error and what was	*
 *	being attempted when the error occurred	*
 *----------------------------------------------*/

void	Check(error, errortype)
	short	error;
	char *	errortype;
{
	Str255	errstring;	/* displayed as "Error <error> occurred while <errstring>" */

	if (error != noErr) {
		NumToString(error, errstring);
		ParamText(errstring, errortype, NIL, NIL);
		if (StopAlert(errAlert,NIL) == 2)	/* Quit button */
			Quit();
	}

}


/*----------------------------------------------*
 *	CopyPString				*
 *						*
 *	Copy a pascal style string to another.	*
 *----------------------------------------------*/

void	CopyPString(fromstr, tostr)
	Str255 *	fromstr;
	Str255 *	tostr;
{
	short	count;

	for (count = (*fromstr)[0]; count >= 0; --count)
		(*tostr)[count] = (*fromstr)[count];	/* includes count byte */
}


/*----------------------------------------------*
 *	CheckNumeric				*
 *						*
 *	Check a pascal style string to make	*
 *	sure it is a valid number. Zero length	*
 *	invalid. Doesn't accept negative.	*
 *----------------------------------------------*/

Boolean	CheckNumeric(str)
	Str255 *	str;
{
	short	count;

	if ((*str)[0] == 0)
		return (FALSE);
	for (count = (*str)[0]; count > 0; --count)
		if (((*str)[count] < '0') || ((*str)[count] > '9'))
			return (FALSE);
	return (TRUE);
}
