/* Copyright (c) 1990 by Steve Peltz */

#include "MacPAD.h"

static	RGBColor	PLATOForeColor,	/* PLATO fore/back colors */
			PLATOBackColor;

void	ClearWindow(void);
Point	Convert(padPt);
short	Mode(void);
void	Color(padRGB *, RGBColor *);


/*----------------------------------------------*
 *	SetTTY, SetPLATO			*
 *						*
 *	These are called when switching between	*
 *	TTY and PLATO modes. When TTY mode is	*
 *	indicated, whatever initializations for	*
 *	TTYChar are required should be done.	*
 *	When PLATO mode is started, the proper	*
 *	screen size should be set up, and any	*
 *	initializations for the PLATO drawing	*
 *	routines should be done. Clearing the	*
 *	screen and resetting colors should also	*
 *	be done.				*
 *----------------------------------------------*/

void	SetTTY()
{
	TTY = TRUE;
	SetPLATOMenu();

	ModeBold = padF;	/* reset plotting parameters */
	Rotate = padF;
	Reverse = padF;
	CurMem = M0;
	CurMode = ModeRewrite;

	PLATOForeColor = Config.DefaultForeColor;
	PLATOBackColor = Config.DefaultBackColor;
	ClearWindow();

	PLATOSize.x = Config.ScreenSize.h;	/* use screen resolution */
	PLATOSize.y = Config.ScreenSize.v;

	CharWide = 8;
	CharHigh = 16;

	TTYLoc.x = 0;
	TTYLoc.y = Config.ScreenSize.v - CharHigh;
}

void	SetPLATO()
{
	TTY = FALSE;
	SetPLATOMenu();

	ClearWindow();

	PLATOSize = Config.PLATOSize;

	CharWide = ((Config.ScreenSize.h * 8) / PLATOSize.x);	/* scale characters */
	CharHigh = ((Config.ScreenSize.v * 16) / PLATOSize.y);
}

void	ClearWindow()
{
	SetPort(mainWindow);
	if (!Mono) {
		RGBForeColor(&PLATOForeColor);
		RGBBackColor(&PLATOBackColor);
	}
	EraseRect(&(mainWindow->portRect));
}


/*----------------------------------------------*
 *	Beep					*
 *						*
 *	Called by PAD to beep terminal.		*
 *----------------------------------------------*/

void	Beep()
{
	SysBeep(5);
}


/*----------------------------------------------*
 *	SendChar				*
 *						*
 *	Called by PAD to send one character	*
 *	out the serial port.			*
 *----------------------------------------------*/

void	SendChar(theKey)
	padByte	theKey;
{
	long	count;
	Byte	bleah;

	if (FlowOff)
		SetFlow(FALSE);
	bleah = theKey;
	do {
		count = 1;
		Check(FSWrite(driverout, &count, &bleah), "\pwriting key");
	} while (count == 0);	/* keep on trying if no room to send */
}


/*----------------------------------------------*
 *	SetFlow					*
 *						*
 *	Called to turn on or off flow control.	*
 *----------------------------------------------*/

void	SetFlow(flow)
	Boolean	flow;
{
	FlowOff = FALSE;
	if (FlowControl)
		if (flow)
			SendChar(0x13);	/* XOF */
		else
			SendChar(0x11);	/* XON */
	FlowOff = flow;
}


/*----------------------------------------------*
 *	MapTouch				*
 *						*
 *	Called by main loop to translate mouse	*
 *	coordinates to PLATO coordinates.	*
 *----------------------------------------------*/

void	MapTouch(pt)
	Point	pt;
{
	padPt	temp;
	Rect	ScreenRect,
		PLATORect;

	SetPort(mainWindow);
	GlobalToLocal(&pt);

	/* do all calcs in long vars to prevent overflow */

	temp.x = ((long)pt.h * PLATOSize.x) / Config.ScreenSize.h;
	temp.y = (PLATOSize.y - 1) - (((long)pt.v * PLATOSize.y) /
		Config.ScreenSize.v);
	
	Touch(temp);
	Beep();
}


/*----------------------------------------------*
 *	Convert					*
 *						*
 *	Local routine to convert a PLATO coord	*
 *	into window coords.			*
 *----------------------------------------------*/

Point	Convert(Coord)
	padPt	Coord;
{
	Point	tempCoord;

	tempCoord.v = ((long)((PLATOSize.y - 1) - Coord.y) * Config.ScreenSize.v)
		/ PLATOSize.y;
	tempCoord.h = ((long)Coord.x * Config.ScreenSize.h) / PLATOSize.x;
	return(tempCoord);
}


/*----------------------------------------------*
 *	Mode					*
 *						*
 *	Local routine to convert a PLATO mode	*
 *	into the appropriate QuickDraw transfer	*
 *	mode.					*
 *----------------------------------------------*/

short	Mode()
{
	if ((CurMode == ModeWrite) || (CurMode == ModeRewrite))
		return (patOr);
	else
		return (patBic);
}


/*----------------------------------------------*
 *	ClearScreen				*
 *						*
 *	Erase the entire window in whichever	*
 *	the current background color is.	*
 *	Setup calls this as well as the PAD.	*
 *----------------------------------------------*/

void	ClearScreen()
{
	SetPort(mainWindow);
	EraseRect(&(mainWindow->portRect));
}


/*----------------------------------------------*
 *	BlockDraw				*
 *						*
 *	Fills or erases rectangular area.	*
 *----------------------------------------------*/

void	BlockDraw(Coord1, Coord2)
	padPt	Coord1,
		Coord2;
{
	Rect	tempRect;
	
	SetPort(mainWindow);
	Pt2Rect(Convert(Coord1), Convert(Coord2), &tempRect);
	tempRect.bottom++;
	tempRect.right++;
	PenMode(Mode());
	PaintRect(&tempRect);
}


/*----------------------------------------------*
 *	CharDraw				*
 *						*
 *	Plot a character at the specified	*
 *	location, in the current plotting	*
 *	modes.					*
 *----------------------------------------------*/

void	CharDraw(Coord, ch, count)
	padPt	Coord;
	padByte	ch[];
	int		count;
{
	Point	Conv;
	short	Mode;
	Point	charSize;
	BitMap	drawMap;
	Byte	drawBits[64][16];
	Byte	*chPt, *drawFrom, *drawTo;
	int	chNum;
	int	direction;
	int	row;
	Rect	fromRect;
	Rect	toRect;
	int	offset;
	
	SetPort(mainWindow);

	drawMap.baseAddr = drawBits;
	drawMap.rowBytes = 64;
	drawMap.bounds.top = 0;
	drawMap.bounds.bottom = 16;
	drawMap.bounds.left = 0;
	drawMap.bounds.right = 512;

	switch (CurMem) {

	case M0:	/* standard ascii chars - first 96 */
		chPt = PCHR;
		offset =  -32;	/* offset for first char in bit map */
		break;

	case M1:	/* standard extended PLATO chars - last 64 */
		chPt = PCHR;
		offset = 64;	/* 96 - 32 */
		break;

	case M2:	/* first half of loaded chars - first 64 */
		chPt = LoadChar;
		offset = -32;	/* offset for first char */
		break;

	case M3:	/* second half of loaded chars - second 64 */
		chPt = LoadChar;
		offset = 32;	/* 64-32 */
		break;
	}
	
	switch (CurMode){

	case ModeWrite:
		Mode = srcOr;
		break;

	case ModeRewrite:
		Mode = srcCopy;
		break;

	case ModeErase:
		Mode = srcBic;
		break;

	case ModeInverse:
		Mode = notSrcCopy;
		break;
	}
	
	if (ModeBold) {
		charSize.h = CharWide + CharWide;
		charSize.v = CharHigh + CharHigh;
	}
	else {
		charSize.h = CharWide;
		charSize.v = CharHigh;
	}

	Conv = Convert(Coord);	

	fromRect.top = 0;
	fromRect.bottom =16;
	fromRect.left = 0;
	fromRect.right = count << 3;

	toRect.bottom = Conv.v + 1;
	toRect.top = toRect.bottom - charSize.v;
	if (Reverse) {
		chNum = count - 1;
		direction = -1;
		toRect.right = Conv.h;
		toRect.left = toRect.right - charSize.h * count;
	}
	else {
		chNum = 0;
		direction = 1;
		toRect.left = Conv.h;
		toRect.right = toRect.left + charSize.h * count;
	}

	while (count--) {
		drawFrom = chPt + ((*ch++ + offset) << 4);
		drawTo = &drawBits[0][chNum];
		chNum += direction;
		for (row = 1; row <= 16; row++) {
			*drawTo = *drawFrom++;
			drawTo += 64;
		}
	}

	CopyBits(&drawMap, &(mainWindow->portBits), &fromRect, &toRect,
		Mode, NIL);

	/* special checks for wrap-around at edges of screen */

	if (toRect.top < 0) {
		toRect.top += Config.ScreenSize.v;
		toRect.bottom += Config.ScreenSize.v;
	CopyBits(&drawMap, &(mainWindow->portBits), &fromRect, &toRect,
		Mode, NIL);
	}

	if (toRect.right > Config.ScreenSize.h) {
		toRect.right -= Config.ScreenSize.h;
		toRect.left -= Config.ScreenSize.h;
		CopyBits(&drawMap, &(mainWindow->portBits), &fromRect, &toRect,
			Mode, NIL);

		if (toRect.bottom > Config.ScreenSize.v) {
			toRect.bottom -= Config.ScreenSize.v;
			toRect.top -= Config.ScreenSize.v;
			CopyBits(&drawMap, &(mainWindow->portBits), &fromRect, &toRect,
				Mode, NIL);
		}
	}
}


/*----------------------------------------------*
 *	TTYChar					*
 *						*
 *	Called to plot chars when in TTY mode.	*
 *----------------------------------------------*/

void	TTYChar(theChar)
	padByte	theChar;
{
	if ((theChar >= 0x20) && (theChar < 0x7F)) {
		CharDraw(TTYLoc, &theChar, 1);
		TTYLoc.x += CharWide;
	}
	else if ((theChar == 0x08) && (TTYLoc.x > 7))	/* backspace */
		TTYLoc.x -= CharWide;
	else if (theChar == 0x0A)			/* line feed */
		TTYLoc.y -= CharHigh;
	else if (theChar == 0x0D)			/* carriage return */
		TTYLoc.x = 0;

	if (TTYLoc.x + CharWide > Config.ScreenSize.h) {	/* wrap at right side */
		TTYLoc.x = 0;
		TTYLoc.y -= CharHigh;
	}

	if (TTYLoc.y < 0) {				/* scroll at bottom */
		SetPort(mainWindow);
		ScrollRect(&(mainWindow->portRect), 0, TTYLoc.y,
			((WindowPeek)mainWindow)->updateRgn);
		SetEmptyRgn(((WindowPeek)mainWindow)->updateRgn);
		TTYLoc.y = 0;
	}
}


/*----------------------------------------------*
 *	CharLoad				*
 *						*
 *	Store a char into a loadable bitmap	*
 *	slot. theChar contains 8x16 bits.	*
 *----------------------------------------------*/

void	CharLoad(charnum, theChar)
	padWord		charnum;
	charData	theChar;
{
	short	bitnum,
		bitmask,
		bitindex,
		column;

	bitmask = 1;
	for (bitnum = 7; bitnum >= 0; bitnum--, bitmask = bitmask << 1) {
		column = theChar[bitnum]; 
		for (bitindex = 15; bitindex >= 0; bitindex--, column = column >> 1)
			if (column & 1)
				LoadChar[charnum][bitindex] |= bitmask;
			else
				LoadChar[charnum][bitindex] &= ~bitmask;
	}
}


/*----------------------------------------------*
 *	DotDraw					*
 *						*
 *	Draw a single dot in the current mode.	*
 *----------------------------------------------*/

void	DotDraw(Coord)
	padPt	Coord;
{
	Point	Conv;

	SetPort(mainWindow);
	Conv = Convert(Coord);
	MoveTo(Conv.h, Conv.v);
	PenMode(Mode());
	Line(0,0);
}


/*----------------------------------------------*
 *	LineDraw				*
 *						*
 *	Draw a line between two points in the	*
 *	current mode.				*
 *----------------------------------------------*/

void	LineDraw(Coord1, Coord2)
	padPt	Coord1,
		Coord2;
{
	Point	Conv;

	SetPort(mainWindow);
	Conv = Convert(Coord1);
	MoveTo(Conv.h,Conv.v);
	Conv = Convert(Coord2);
	PenMode(Mode());
	LineTo(Conv.h,Conv.v);
}


/*----------------------------------------------*
 *	Color					*
 *						*
 *	Local routine to translate PLATO colors	*
 *	to Mac colors.				*
 *----------------------------------------------*/

void	Color(theColor, newColor)
	padRGB *	theColor;
	RGBColor *	newColor;
{
	newColor->red = (theColor->red << 8) | ((theColor->red & 0x80)? 0xFF : 0x00);
	newColor->green = (theColor->green << 8) | ((theColor->green & 0x80)? 0xFF : 0x00);
	newColor->blue = (theColor->blue << 8) | ((theColor->blue & 0x80)? 0xFF : 0x00);
}


/*----------------------------------------------*
 *	ForeGround				*
 *						*
 *	Set the foreground color.		*
 *----------------------------------------------*/

void	ForeGround(theColor)
	padRGB	*theColor;
{
	if (!Mono) {
		Color(theColor, &PLATOForeColor);
		SetPort(mainWindow);
		RGBForeColor(&PLATOForeColor);
	}
}


/*----------------------------------------------*
 *	BackGround				*
 *						*
 *	Set the background color.		*
 *----------------------------------------------*/

void	BackGround(theColor)
	padRGB *theColor;
{
	if (!Mono) {				
		Color(theColor, &PLATOBackColor);
		SetPort(mainWindow);
		RGBBackColor(&PLATOBackColor);
	}
}


/*----------------------------------------------*
 *	SetFlowControl				*
 *						*
 *	Called to switch to flow control mode	*
 *	if it is ok (or turn it off).		*
 *----------------------------------------------*/

padBool	SetFlowControl(allow)
	padBool	allow;
{
	if (FlowOff && (!allow))
		SetFlow(FALSE);
	FlowControl = Config.FlowControl & allow;
	SetFlowMenu();
}


/*----------------------------------------------*
 *	CharAddress, TermType, SubType, etc.	*
 *						*
 *	Called to return config info. 		*
 *----------------------------------------------*/

padByte	TermType()
{
	return(Config.TermType);
}

padByte	SubType()
{
	return(Config.SubType);
}

padByte	LoadFile()
{
	return(Config.LoadFile);
}

padByte	Configuration()
{
	return(Config.Configuration);
}

padWord	CharAddress()
{
	return(Config.CharLoadAddress);
}


/*----------------------------------------------*
 *	TouchAllow				*
 *						*
 *	Sets whether touch is to be active.	*
 *----------------------------------------------*/

void	TouchAllow(allow)
	padBool	allow;
{
	TouchActive = allow;
}


/*----------------------------------------------*
 *	ExtAllow				*
 *						*
 *	External input allowed. Not implemented.*
 *----------------------------------------------*/

void	ExtAllow(allow)
	padBool	allow;
{
}


/*----------------------------------------------*
 *	SetExtIn				*
 *						*
 *	Set which device to get input from.	*
 *	Not implemented.			*
 *----------------------------------------------*/

void	SetExtIn(device)
	padWord	device;
{
}


/*----------------------------------------------*
 *	SetExtOut				*
 *						*
 *	Set which device to send external data	*
 *	to. Not implemented.			*
 *----------------------------------------------*/

void	SetExtOut(device)
	padWord	device;
{
}


/*----------------------------------------------*
 *	ExtIn					*
 *						*
 *	Get a byte of data from external device	*
 *	(even if there isn't any).		*
 *	Not implemented.			*
 *----------------------------------------------*/

padByte	ExtIn()
{
	return(0x00);
}


/*----------------------------------------------*
 *	ExtOut					*
 *						*
 *	Send a byte of data to external device.	*
 *	Not implemented.			*
 *----------------------------------------------*/

void	ExtOut(value)
	padByte	value;
{
}


/*----------------------------------------------*
 *	MemLoad					*
 *						*
 *	Load program memory with 16 bit data.	*
 *	Not implemented.			*
 *----------------------------------------------*/

void	MemLoad(addr, value)
	padWord	addr,
		value;
{
}


/*----------------------------------------------*
 *	MemRead					*
 *						*
 *	Read a byte of program memory.		*
 *	Not implemented.			*
 *----------------------------------------------*/

padByte	MemRead(addr)
	padWord	addr;
{
	return(0xFF);
}


/*----------------------------------------------*
 *	Mode5, Mode6, Mode7			*
 *						*
 *	User programmable data modes.		*
 *	Not implemented.			*
 *----------------------------------------------*/

void	Mode5(value)
	padWord	value;
{
}

void	Mode6(value)
	padWord	value;
{
}

void	Mode7(value)
	padWord	value;
{
}
