/* Copyright (c) 1990 by Steve Peltz */

#include "MacPAD.h"
#include <SerialDvr.h>
#include <MemoryMgr.h>
#include <MenuMgr.h>
#include <OSUtil.h>

void	NewPLATOWindow(void);
void	OpenPLATOWindow(void);

#define myBarID		128
#define KtPID		128
#define KtPName		'KTPT'


/*----------------------------------------------*
 *	InitAll					*
 *						*
 *	Initialize Mac toolbox, set up menu	*
 *	bar, initialize serial port, open main	*
 *	window, initialize variables, and plot	*
 *	copyright notice.			*
 *----------------------------------------------*/

void	InitAll(bufferSize)
	short	bufferSize;
{
	SysEnvRec	environment;
	Handle		myBar;
	Ptr		buffer;
	SerShk		serstatus;

	InitGraf(&thePort);
	InitFonts();
	InitWindows();
	InitMenus();
	TEInit();
	InitDialogs(Quit);
	InitCursor();

	if (SysEnvirons(1, &environment) == noErr)
		Mono = !environment.hasColorQD;
	else
		Mono = TRUE;
	
	myBar = GetNewMBar(myBarID);			/* set up menu bar */
	SetMenuBar(myBar);
	DrawMenuBar();
	DisposHandle(myBar);

	OpenFinderFile();

	Check(OpenDriver("\p.AOut", &driverout), "\popening output driver");
	Check(OpenDriver("\p.AIn", &driverin), "\popening input driver");

	serstatus.fXOn = 0;
	serstatus.fCTS = 0;
	serstatus.errs = 0;
	serstatus.evts = 0;
	serstatus.fInX = 0;

	Check(SerHShake(driverin, &serstatus), "\psetting handshake");
	BaudRate(Config.BaudRate);

	buffer = NewPtr(bufferSize);
	Check(MemError(), "\pgetting serial buffer memory");

	Check(SerSetBuf(driverin, buffer, bufferSize), "\psetting input buffer");

	KtP = (KtPTab **)GetResource(KtPName, KtPID);
	Check(ResError(), "\pgetting Key-to-PLATO translate table");

	OpenPLATOWindow();
	InitPAD();
	NewPLATOWindow();
}


/*----------------------------------------------*
 *	NewPLATOWindow				*
 *						*
 *	Plot copyright notice.			*
 *----------------------------------------------*/

void	NewPLATOWindow()
{
	Rect	tempRect;

	SetRect(&tempRect, (Config.ScreenSize.h / 2) - 111, 15,
		(Config.ScreenSize.h / 2) + 111, 60);
	PaintRoundRect(&tempRect, 10, 10);
	TextMode(srcBic);
	MoveTo((Config.ScreenSize.h / 2) - 51, 30);
	DrawString("\pMacPAD Version 0.8");
	MoveTo((Config.ScreenSize.h / 2) -99, 50);
	DrawString("\pCopyright ");
	DrawChar(0xA9); /* Copyright symbol */
	DrawString("\p 1990 by Steve Peltz");
	TTYLoc.y = Config.ScreenSize.v - 10 * CharHigh;	/* set below copyright notice */
}


/*----------------------------------------------*
 *	OpenPLATOWindow				*
 *						*
 *	Open new PLATO window.			*
 *----------------------------------------------*/

void	OpenPLATOWindow()
{
	Rect	tempRect;

	topLeft(tempRect) = WindowLoc;
	tempRect.top += menuOffset + (WindowType != plainWindow) * titleOffset;
	tempRect.bottom = tempRect.top + Config.ScreenSize.v;
	tempRect.right = tempRect.left + Config.ScreenSize.h;

	if (Mono)
		mainWindow = NewWindow(NIL, &tempRect,
			&ConfigFile, TRUE, WindowType,
			Front, FALSE, 0);
	else
		mainWindow = (WindowPtr)NewCWindow(NIL, &tempRect,
			&ConfigFile, TRUE, WindowType,
			Front, FALSE, 0);
}


/*----------------------------------------------*
 *	ResetPLATOWindow			*
 *						*
 *	Close old window and open new one,	*
 *	presumably after changing parameters.	*
 *----------------------------------------------*/

void	ResetPLATOWindow()
{
	CloseWindow(mainWindow);
	OpenPLATOWindow();
	if (TTY) {
		SetTTY();
		NewPLATOWindow();
	}
	else
		SetPLATO();
}


/*----------------------------------------------*
 *	Quit					*
 *						*
 *	Quit is called by the main loop to exit	*
 *	or by the system error handler (see the	*
 *	call to InitDialogs in Setup) if a	*
 *	system error occurs. It is also called	*
 *	if Check detects an error. It does	*
 *	anything necessary before leaving.	*
 *	Resetting the serial buffer is		*
 *	necessary to prevent system errors.	*
 *	Closing the serial port can disable the	*
 *	mouse, so we leave it open. It would	*
 *	also drop DTR, which might be nice.	*
 *----------------------------------------------*/

void	Quit()
{
	short dummy;

	dummy = SerSetBuf(driverin, NIL, 0);
	ExitToShell();
}


/*----------------------------------------------*
 *	BaudRate				*
 *						*
 *	BaudRate is called to set the serial	*
 *	port to the specified baud rate.	*
 *----------------------------------------------*/

void	BaudRate(theItem)
	short	theItem;
{
	short	Baud;
	short	serialparam;

	switch (theItem) {
		case 0:	Baud = baud300;		break;
		case 1:	Baud = baud1200;	break;
		case 2:	Baud = baud2400;	break;
		case 3: Baud = baud4800;	break;
		case 4:	Baud = baud9600;	break;
	}
	serialparam = Baud + stop10 + evenParity + data7;
	Check(SerReset(driverout, serialparam), "\psetting baud rate");
}
