/* Copyright (c) 1990 by Steve Peltz */

#include "MacPAD.h"
#include <MenuMgr.h>
#include <EventMgr.h>

#define appleMenu	128
#define about		1

#define fileMenu	129
#define new		1
#define open		2
#define save		3
#define saveas		4
#define quit		5

#define editMenu	130

#define confMenu	131
#define forceTTY	1
#define forcePLATO	2
#define flowon		3
#define configure	5
#define cmdcontrol	6

#define bit(x,y)	(x[y >> 3] >> (y & 7)) & 1

static	Boolean	 	CommandEnable,	/* command key equivalents */
			done;		/* flag to quit from menu routine */
			
/* global variables */

	WindowPtr	mainWindow;

	short		driverin,	/* port driver identifiers */
			driverout;

	KtPPtr *	KtP;		/* handle to keyboard to PLATO table */

	CharBits	LoadChar[128];	/* Loadable charset data */

	short		CharWide,	/* current width and height of screen charset */
			CharHigh;

	padPt		TTYLoc;		/* coordinate for TTYChar */

	Boolean		Mono,		/* mono or color */
			FlowOff,	/* whether xoff has been sent */
			TouchActive;	/* should only be checked in PLATO mode */

	Point		WindowLoc;	/* current window location */
	short		WindowType;	/* current window type */

	ConfigRec	Config;
	Str255		ConfigFile;
	short		ConfigVRefNum;

void	main(void);
void	MouseClick(EventRecord *);
void	DragPLATOWindow(Point, WindowPtr);
void	KeyPress(EventRecord *);
void	DoCommand(long);

#define BufferSize	1024
#define Threshold	256
#define	ProcessSize	64


void	main()
{
	short		dummy;
	long		count;
	padByte		theBytes[ProcessSize];
	EventRecord	theEvent;

	InitAll(BufferSize);
	AddResMenu(GetMHandle(appleMenu), 'DRVR');	/* add on apple menu items */
	CommandEnable = TRUE;
	
	FlowOff = FALSE;
	done = FALSE;
	while (!done) {
		dummy = SerGetBuf(driverin, &count);	/* bug in old driver gives error status */
		if ((dummy == 0) && (count > 0)) {	/* so ignore return if error */
			if ((!FlowOff) && (count > (BufferSize - Threshold)))
				SetFlow(TRUE);		/* buffer about to overflow */
			else if ((FlowOff) && (count < Threshold))
				SetFlow(FALSE);
			if (count > ProcessSize)	/* limit to max we want */
				count = ProcessSize;
			dummy = FSRead(driverin, &count, theBytes);
			if (dummy == 0)			/* damn erroneous error codes */
				ShowPLATO(theBytes, (short)count);
		}
		SystemTask();				/* give time to DAs */
		if (GetNextEvent(everyEvent, &theEvent)) {
			switch (theEvent.what) {
			case mouseDown:
				MouseClick(&theEvent);
				break;
			case keyDown:
			case autoKey:
				KeyPress(&theEvent);
				break;
			}
		}
	}
	Quit();
}

void	MouseClick(theEvent)
	EventRecord *	theEvent;
{
	WindowPtr	theWindow;

	switch (FindWindow(theEvent->where, &theWindow)) {

	case inMenuBar:
		DoCommand(MenuSelect(theEvent->where));
		break;

	case inSysWindow:
		SystemClick(theEvent, theWindow);
		break;

	case inDrag:
		DragPLATOWindow(theEvent->where, theWindow);
		break;

	case inContent:
		if (theEvent->modifiers & cmdKey)
			DragPLATOWindow(theEvent->where, theWindow);
		else if (FrontWindow() != theWindow)
			SelectWindow(theWindow);
		else if (TouchActive) {
			MapTouch(theEvent->where);
		}
		break;
	}
}

void	DragPLATOWindow(where, theWindow)
	Point		where;
	WindowPtr	theWindow;
{
	Rect	boundsRect;

	boundsRect.top = screenBits.bounds.top - 24;
	boundsRect.bottom = screenBits.bounds.bottom + 4;
	boundsRect.left = screenBits.bounds.left + 4;
	boundsRect.right = screenBits.bounds.right - 4;
	DragWindow(theWindow, where, &boundsRect);
	SetPort(theWindow);
	WindowLoc = topLeft(theWindow->portRect);
	LocalToGlobal(&WindowLoc);
	WindowLoc.v -= menuOffset + (WindowType != plainWindow) * titleOffset;
}

void	KeyPress(theEvent)
	EventRecord *	theEvent;
{
	Boolean		commandKey,
			shftKey;
	short		theValue;

	if ((theEvent->modifiers & optionKey) == 0) {
		commandKey = (theEvent->modifiers & cmdKey) != 0;
		if (TTY) {
				theValue = (theEvent->message & 0x7f);
				if (commandKey && CommandEnable)
					DoCommand(MenuKey(theValue));
				else {
					ObscureCursor();
					if (commandKey && (
					((theValue >= '@') && (theValue <= '_')) ||
					((theValue >= 'a') && (theValue <= 'z'))))
						theValue &= 0x1f;
					SendChar(theValue);
				}
		}
		else {
			theValue = (theEvent->message & keyCodeMask) >> 8;
			shftKey = (theEvent->modifiers & shiftKey) != 0;
			theValue = (*KtP)->KeyCode[theValue][commandKey][shftKey];
			if (theValue == 0xff) { 
				if (commandKey && (theEvent->what == keyDown))
					DoCommand(MenuKey(theEvent->message & charCodeMask));
			}
			else {
				if (!TouchActive)
					ObscureCursor();
				if (theValue & 0x80) {
					Key(ACCESS);
					Key((*KtP)->Special[theValue & 0x7f]);
				}
				else if ((theEvent->what == keyDown)
				|| (bit((*KtP)->Repeat, theValue))) {
					if (theEvent->modifiers & controlKey) {
						if (bit((*KtP)->Access, theValue)) {
							Key(ACCESS);	/* access key code */
							Key(theValue);
						}
					}
					else {
						if ((theEvent->modifiers & alphaLock) &&
						(theValue >= 0x41) &&
						(theValue <= 0x5a))
							theValue |= 0x20;
						Key(theValue);
					}
				}
			}
		}
	}
}

void	DoCommand(selection)
	long	selection;
{
	short		theMenu;
	short		theItem;
	Str255		theName;

	theMenu = HiWord(selection);
	theItem = LoWord(selection);
	switch (theMenu) {

	case appleMenu:
		if (theItem == about)
			About();
		else {
			GetItem(GetMHandle(theMenu), theItem, &theName);
			OpenDeskAcc(&theName);
		}
		break;

	case fileMenu:
		switch (theItem) {
		case new:
			if (AskConfigFile()) {
				DefaultConfig();
				ResetPLATOWindow();
				BaudRate(Config.BaudRate);
			}
			break;
		case open:
			GetConfigFile();
			BaudRate(Config.BaudRate);
			break;
		case save:
			SaveConfigFile(FALSE);
			break;
		case saveas:
			SaveConfigFile(TRUE);
			break;
		case quit:
			done = AskConfigFile();
			break;
		}
		break;

	case editMenu:
		SystemEdit(theItem - 1);
		break;

	case confMenu:
		switch (theItem) {
		case forcePLATO:
			if (TTY)
				ClearScreen();
			InitPLATO();
			break;

		case forceTTY:
			InitTTY();	/* clears screen for us */
			break;

		case flowon:
			FlowControl = !FlowControl;
			SetFlowMenu();
			break;

		case configure:
			Configure();
			break;

		case cmdcontrol:
			CheckItem(GetMHandle(theMenu), theItem, CommandEnable);
			CommandEnable = !CommandEnable;
			break;
		}
		break;
	}
	HiliteMenu(0);
}

void	SetPLATOMenu()
{
	MenuHandle	menu;

	menu = GetMHandle(confMenu);
	CheckItem(menu, forcePLATO, !TTY);
	CheckItem(menu, forceTTY, TTY);
	if (TTY)
		DisableItem(menu, flowon);
	else
		EnableItem(menu, flowon);
}

void	SetFlowMenu()
{
	CheckItem(GetMHandle(confMenu), flowon, FlowControl);
}
